﻿using System;
using Xamarin.Forms;
using C1.Xamarin.Forms.Chart;
using FlexChart101.Resources;
using Xamarin.Forms.Xaml;
using C1.Xamarin.Forms.Chart.Interaction;
using System.Collections.Generic;

namespace FlexChart101
{
    [XamlCompilation(XamlCompilationOptions.Compile)]
    public partial class ToggleSeriesSample
    {
        IEnumerable<SalesExpensesDownloadsEntity> list;
        public ToggleSeriesSample()
        {
            InitializeComponent();
            Title = AppResources.ToggleSeriesTitle;
            list = ChartSampleFactory.CreateEntityList();
            this.flexChart.ItemsSource = list;
            this.flexChart.Palette = Palette.Darkly;
            this.flexChart.AxisX.Format = "M/dd";
            ZoomBehavior z = new ZoomBehavior();
            z.ZoomMode = GestureMode.X;
            flexChart.Behaviors.Add(z);
            
            TranslateBehavior t = new TranslateBehavior();
            flexChart.Behaviors.Add(t);
            flexChart.Rendered += FlexChart_Rendered;
        }

        private void FlexChart_Rendered(object sender, RenderEventArgs e)
        {
            IChartAxis chartAxis = flexChart.AxisX as IChartAxis;
            DateTime dateMin = DateTime.FromOADate(chartAxis.GetMin());
            DateTime dateMax = DateTime.FromOADate(chartAxis.GetMax());
            double actualYMin = double.MaxValue;
            double actualYMax = double.MinValue;
            foreach (SalesExpensesDownloadsEntity entity in list)
            {
                if (entity.Date >= dateMin && entity.Date <= dateMax)
                {
                    if (actualYMin > entity.Sales)
                    {
                        actualYMin = entity.Sales;
                    }
                    if (actualYMax < entity.Sales)
                    {
                        actualYMax = entity.Sales;
                    }
                }
            }
            flexChart.AxisY.Min = actualYMin;
            flexChart.AxisY.Max = actualYMax;
        }
    }
    public class SeriesVisibilityBoolConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            ChartSeriesVisibilityType visible = (ChartSeriesVisibilityType)value;
            return visible != ChartSeriesVisibilityType.Hidden && visible != ChartSeriesVisibilityType.Legend;
           
        }

        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            bool val = (bool)value;
            return val ? ChartSeriesVisibilityType.Visible : ChartSeriesVisibilityType.Hidden;
        }
    }

}
